"""
Test reporter.
"""
import unittest
import datetime
import pytest
import ddt

from enterprise_reporting import reporter
from enterprise_reporting.reporter import EnterpriseReportSender
from enterprise_reporting.utils import encrypt_string


@ddt.ddt
class TestReporter(unittest.TestCase):
	""""
	Tests about reporter methods
	"""
	def setUp(self):
		self.FILE_WRITE_DIRECTORY = '/tmp'
		self.date = datetime.datetime.now().strftime("%Y-%m-%d")
		self.reporting_config = {
			'enterprise_customer': {
				'uuid': '4815162242', 'name': 'John'
			},
			'delivery_method': 'sftp',
			'data_type': 'catalog',
			'uuid': '987654321',
			'sftp_hostname': '127.0.0.1',
			'active': True,
			'report_type': 'json',
			'encrypted_password': 'bleeH-bLah-blUe001',
			'sftp_username': 'John1',
			'sftp_port': 22,
			'sftp_file_path': 'home/user/Documents',
			'encrypted_sftp_password': 'abraAcaDabradOo',
		}
		sftp_password = self.reporting_config['encrypted_sftp_password']
		self.reporting_config['encrypted_sftp_password'] = encrypt_string(sftp_password)

	@ddt.data(
		True,
		False
	)
	def test_data_report_file_name(self, flag_value):
		"""
		Tests if the report config file name is generated correctly when date is included
		"""
		self.reporting_config['include_date'] = flag_value
		enterprise_report_sender = EnterpriseReportSender.create(self.reporting_config)
		actual_file_name = enterprise_report_sender.data_report_file_name
		date_str = f"_{self.date}" if self.reporting_config.get('include_date') else ""
		expected_file_name = "{dir}/{enterprise_uuid}_{data_type}_{report_type}{date}.{report_type}".format(
			dir=self.FILE_WRITE_DIRECTORY,
			enterprise_uuid=self.reporting_config['enterprise_customer']['uuid'],
			data_type=self.reporting_config['data_type'],
			date=date_str,
			report_type=self.reporting_config['report_type']
		)
		assert actual_file_name == expected_file_name



"""
Tests for the `enterprise-data` models module.
"""


import unittest

import ddt
from pytest import mark

from enterprise_data_roles.tests import factories


@mark.django_db
@ddt.ddt
class TestEnterpriseDataFeatureRole(unittest.TestCase):
    """
    Tests for Enterprise Data Feature Role model
    """

    def setUp(self):
        self.enterprise_data_feature_role = factories.EnterpriseDataFeatureRoleFactory(name="Test Role")
        self.enterprise_data_role_assignment = factories.EnterpriseDataRoleAssignmentFactory()

        super().setUp()

    @ddt.data(str, repr)
    def test_string_conversion(self, method):
        """
        Test conversion to string.
        """
        expected_str = "EnterpriseDataFeatureRole(name=Test Role)"
        assert expected_str == method(self.enterprise_data_feature_role)


@mark.django_db
@ddt.ddt
class TestEnterpriseDataRoleAssignment(unittest.TestCase):
    """
    Tests for Enterprise Data Role Assignment model
    """

    def setUp(self):
        self.enterprise_data_role_assignment = factories.EnterpriseDataRoleAssignmentFactory(
            role__name="Test Role",
            user__id=1,
        )

        super().setUp()

    @ddt.data(str, repr)
    def test_string_conversion(self, method):
        """
        Test conversion to string.
        """
        expected_str = "EnterpriseDataRoleAssignment(name=Test Role, user=1)"
        assert expected_str == method(self.enterprise_data_role_assignment)



"""
URL definitions for enterprise data API endpoint.
"""


from django.conf.urls import include, url

urlpatterns = [
    url(r'^v0/', include('enterprise_data.api.v0.urls', 'v0'), name='api_v0'),
    url(r'^v1/', include('enterprise_data.api.v1.urls', 'v1'), name='api_v1'),
]



# Generated by Django 1.11.5 on 2018-05-08 16:52


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data', '0004_auto_20180501_0928'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterpriseenrollment',
            name='course_key',
            field=models.CharField(max_length=255, null=True),
        ),
    ]



"""
Serializers for enterprise api v1.
"""


from rest_framework import serializers

from enterprise_data.models import EnterpriseLearner, EnterpriseLearnerEnrollment


class EnterpriseLearnerEnrollmentSerializer(serializers.ModelSerializer):
    """
    Serializer for EnterpriseLearnerEnrollment model.
    """
    course_api_url = serializers.SerializerMethodField()
    enterprise_user_id = serializers.SerializerMethodField()

    class Meta:
        model = EnterpriseLearnerEnrollment
        # Do not change the order of fields below. Ordering is important becuase `progress_v3`
        # csv generated in `enterprise_reporting` should be same as csv generated on `admin-portal`
        # Order and field names below should match with `EnterpriseLearnerEnrollmentViewSet.header`
        fields = (
            'enrollment_id', 'enterprise_enrollment_id', 'is_consent_granted', 'paid_by',
            'user_current_enrollment_mode', 'enrollment_date', 'unenrollment_date',
            'unenrollment_end_within_date', 'is_refunded', 'seat_delivery_method',
            'offer_name', 'offer_type', 'coupon_code', 'coupon_name', 'contract_id',
            'course_list_price', 'amount_learner_paid', 'course_key', 'courserun_key',
            'course_title', 'course_pacing_type', 'course_start_date', 'course_end_date',
            'course_duration_weeks', 'course_max_effort', 'course_min_effort',
            'course_primary_program', 'course_primary_subject', 'has_passed',
            'last_activity_date', 'progress_status', 'passed_date', 'current_grade',
            'letter_grade', 'enterprise_user_id', 'user_email', 'user_account_creation_date',
            'user_country_code', 'user_username', 'enterprise_name', 'enterprise_customer_uuid',
            'enterprise_sso_uid', 'created', 'course_api_url',
        )

    def get_course_api_url(self, obj):
        """Constructs course api url"""
        return '/enterprise/v1/enterprise-catalogs/{enterprise_customer_uuid}/courses/{courserun_key}'.format(
            enterprise_customer_uuid=obj.enterprise_customer_uuid, courserun_key=obj.courserun_key
        )

    def get_enterprise_user_id(self, obj):
        """Returns enterprise user id of a learner's enrollment"""
        return obj.enterprise_user_id


class EnterpriseLearnerSerializer(serializers.ModelSerializer):
    """
    Serializer for EnterpriseLearner model.
    """

    class Meta:
        model = EnterpriseLearner
        fields = '__all__'

    def to_representation(self, instance):
        representation = super().to_representation(instance)

        if hasattr(instance, 'enrollment_count'):
            representation['enrollment_count'] = instance.enrollment_count
        if hasattr(instance, 'course_completion_count'):
            representation['course_completion_count'] = instance.course_completion_count

        return representation


class LearnerCompletedCoursesSerializer(serializers.Serializer):    # pylint: disable=abstract-method
    """
    Serializer for learner's completed courses.
    """
    class Meta:
        ref_name = 'v1.LearnerCompletedCoursesSerializer'

    user_email = serializers.EmailField()
    completed_courses = serializers.IntegerField()



# Generated by Django 2.2.17 on 2021-07-15 13:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data_roles', '0006_remove_role_based_access_control_switch'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisedataroleassignment',
            name='applies_to_all_contexts',
            field=models.BooleanField(default=False, help_text='If true, indicates that the user is effectively assigned their role for any and all contexts. Defaults to False.'),
        ),
    ]



"""
URL definitions for enterprise data api v1.
"""


from rest_framework.routers import DefaultRouter

from enterprise_data.api.v1 import views

app_name = 'enterprise_data_api_v1'

router = DefaultRouter()
router.register(
    r'enterprise/(?P<enterprise_id>.+)/enrollments',
    views.EnterpriseLearnerEnrollmentViewSet,
    'enterprise-learner-enrollment',
)
router.register(
    r'enterprise/(?P<enterprise_id>.+)/users',
    views.EnterpriseLearnerViewSet,
    'enterprise-learner',
)
router.register(
    r'enterprise/(?P<enterprise_id>.+)/learner_completed_courses',
    views.EnterpriseLearnerCompletedCoursesViewSet,
    'enterprise-learner-completed-courses',
)

urlpatterns = router.urls
